from .test_callbacks import TestCallbacks as callbacks
from .test_codestream import TestCodestream as codestream
from .test_config import TestSuite as config
from .test_jp2k import TestJp2k as jp2k
from .test_icc import TestICC as icc
from .test_printing import TestPrinting as printing
from .test_opj_suite import TestSuite as suite
from .test_opj_suite import TestSuiteDump as suitedump
from .test_opj_suite_write import TestSuiteWrite as suitew
from .test_opj_suite_neg import TestSuiteNegative as suiteneg
from .test_jp2box import TestJp2Boxes as box



import re
import sys

import numpy as np


def mse(amat, bmat):
    """Mean Square Error"""
    diff = amat.astype(np.double) - bmat.astype(np.double)
    err = np.mean(diff**2)
    return err


def peak_tolerance(amat, bmat):
    """Peak Tolerance"""
    diff = np.abs(amat.astype(np.double) - bmat.astype(np.double))
    ptol = diff.max()
    return ptol


def read_pgx(pgx_file):
    """Helper function for reading the PGX comparison files.

    Open the file in ascii mode and read the header line.
    Will look something like

    PG ML + 8 128 128
    PG%[ \t]%c%c%[ \t+-]%d%[ \t]%d%[ \t]%d"
    """
    header = ''
    with open(pgx_file, 'rb') as fptr:
        while True:
            char = fptr.read(1)
            if char[0] == 10 or char == '\n':
                pos = fptr.tell()
                break
            else:
                if sys.hexversion < 0x03000000:
                    header += char
                else:
                    header += chr(char[0])

    header = header.rstrip()
    tokens = re.split('\s', header)

    if (tokens[1][0] == 'M') and (sys.byteorder == 'little'):
        swapbytes = True
    elif (tokens[1][0] == 'L') and (sys.byteorder == 'big'):
        swapbytes = True
    else:
        swapbytes = False

    if (len(tokens) == 6):
        bitdepth = int(tokens[3])
        signed = bitdepth < 0
        if signed:
            bitdepth = -1 * bitdepth
        nrows = int(tokens[5])
        ncols = int(tokens[4])
    else:
        bitdepth = int(tokens[2])
        signed = bitdepth < 0
        if signed:
            bitdepth = -1 * bitdepth
        nrows = int(tokens[4])
        ncols = int(tokens[3])

    if signed:
        if bitdepth <= 8:
            dtype = np.int8
        elif bitdepth <= 16:
            dtype = np.int16
        else:
            raise RuntimeError("unhandled bitdepth")
    else:
        if bitdepth <= 8:
            dtype = np.uint8
        elif bitdepth <= 16:
            dtype = np.uint16
        else:
            raise RuntimeError("unhandled bitdepth")

    shape = [nrows, ncols]

    # Reopen the file in binary mode and seek to the start of the binary
    # data
    with open(pgx_file, 'rb') as fptr:
        fptr.seek(pos)
        data = np.fromfile(file=fptr, dtype=dtype).reshape(shape)

    return(data.byteswap(swapbytes))



#pylint:  disable-all
import doctest
import os
import pkg_resources
import shutil
import struct
import sys
import tempfile

if sys.hexversion < 0x02070000:
    import unittest2 as unittest
else:
    import unittest

import numpy as np

import glymur


@unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
@unittest.skipIf(glymur.lib._openjp2.OPENJP2 is None,
                 "Missing openjp2 library.")
class TestOpenJP2(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_set_default_encoder_parameters(self):
        cparams = glymur.lib._openjp2.set_default_encoder_parameters()

        self.assertEqual(cparams.res_spec, 0)
        self.assertEqual(cparams.cblockw_init, 64)
        self.assertEqual(cparams.cblockh_init, 64)
        self.assertEqual(cparams.numresolution, 6)
        self.assertEqual(cparams.subsampling_dx, 1)
        self.assertEqual(cparams.subsampling_dy, 1)
        self.assertEqual(cparams.mode, 0)
        self.assertEqual(cparams.prog_order, glymur.core.LRCP)
        self.assertEqual(cparams.roi_shift, 0)
        self.assertEqual(cparams.cp_tx0, 0)
        self.assertEqual(cparams.cp_ty0, 0)

        self.assertEqual(cparams.irreversible, 0)

    def test_set_default_decoder_parameters(self):
        dparams = glymur.lib._openjp2.set_default_decoder_parameters()

        self.assertEqual(dparams.DA_x0, 0)
        self.assertEqual(dparams.DA_y0, 0)
        self.assertEqual(dparams.DA_x1, 0)
        self.assertEqual(dparams.DA_y1, 0)

    def tile_macro(self, codec, stream, imagep, tidx):
        # called only by j2k_random_tile_access
        glymur.lib._openjp2.get_decoded_tile(codec, stream, imagep, tidx)
        for j in range(imagep.contents.numcomps):
            self.assertIsNotNone(imagep.contents.comps[j].data)

    def j2k_random_tile_access(self, filename, codec_format=None):
        # called by the test_rtaX methods
        dparam = glymur.lib._openjp2.set_default_decoder_parameters()

        infile = filename.encode()
        nelts = glymur.lib._openjp2.PATH_LEN - len(infile)
        infile += b'0' * nelts
        dparam.infile = infile

        dparam.decod_format = codec_format

        codec = glymur.lib._openjp2.create_decompress(codec_format)

        glymur.lib._openjp2.set_info_handler(codec, None)
        glymur.lib._openjp2.set_warning_handler(codec, None)
        glymur.lib._openjp2.set_error_handler(codec, None)

        x = (filename, True)
        stream = glymur.lib._openjp2.stream_create_default_file_stream_v3(*x)

        glymur.lib._openjp2.setup_decoder(codec, dparam)
        image = glymur.lib._openjp2.read_header(stream, codec)

        cstr_info = glymur.lib._openjp2.get_cstr_info(codec)

        tile_ul = 0
        tile_ur = cstr_info.contents.tw - 1
        tile_lr = cstr_info.contents.tw * cstr_info.contents.th - 1
        tile_ll = tile_lr - cstr_info.contents.tw

        self.tile_macro(codec, stream, image, tile_ul)
        self.tile_macro(codec, stream, image, tile_ur)
        self.tile_macro(codec, stream, image, tile_lr)
        self.tile_macro(codec, stream, image, tile_ll)

        glymur.lib._openjp2.destroy_cstr_info(cstr_info)

        glymur.lib._openjp2.end_decompress(codec, stream)
        glymur.lib._openjp2.destroy_codec(codec)
        glymur.lib._openjp2.stream_destroy_v3(stream)
        glymur.lib._openjp2.image_destroy(image)

    def tile_decoder(self, x0=None, y0=None, x1=None, y1=None, filename=None,
                     codec_format=None):
        x = (filename, True)
        stream = glymur.lib._openjp2.stream_create_default_file_stream_v3(*x)
        dparam = glymur.lib._openjp2.set_default_decoder_parameters()

        dparam.decod_format = codec_format

        # Do not use layer decoding limitation.
        dparam.cp_layer = 0

        # do not use resolution reductions.
        dparam.cp_reduce = 0

        codec = glymur.lib._openjp2.create_decompress(codec_format)

        glymur.lib._openjp2.set_info_handler(codec, None)
        glymur.lib._openjp2.set_warning_handler(codec, None)
        glymur.lib._openjp2.set_error_handler(codec, None)

        glymur.lib._openjp2.setup_decoder(codec, dparam)
        image = glymur.lib._openjp2.read_header(stream, codec)
        glymur.lib._openjp2.set_decode_area(codec, image, x0, y0, x1, y1)

        data = np.zeros((1150, 2048, 3), dtype=np.uint8)
        while True:
            rargs = glymur.lib._openjp2.read_tile_header(codec, stream)
            tidx = rargs[0]
            sz = rargs[1]
            go_on = rargs[-1]
            if not go_on:
                break
            glymur.lib._openjp2.decode_tile_data(codec, tidx, data, sz, stream)

        glymur.lib._openjp2.end_decompress(codec, stream)
        glymur.lib._openjp2.destroy_codec(codec)
        glymur.lib._openjp2.stream_destroy_v3(stream)
        glymur.lib._openjp2.image_destroy(image)

    def tile_encoder(self, num_comps=None, tile_width=None, tile_height=None,
                     filename=None, codec=None, comp_prec=None,
                     image_width=None, image_height=None,
                     irreversible=None):
        num_tiles = (image_width / tile_width) * (image_height / tile_height)
        tile_size = tile_width * tile_height * num_comps * comp_prec / 8

        data = np.random.random((tile_height, tile_width, num_comps))
        data = (data * 255).astype(np.uint8)

        l_param = glymur.lib._openjp2.set_default_encoder_parameters()

        l_param.tcp_numlayers = 1
        l_param.cp_fixed_quality = 1
        l_param.tcp_distoratio[0] = 20

        # position of the tile grid aligned with the image
        l_param.cp_tx0 = 0
        l_param.cp_ty0 = 0

        # tile size, we are using tile based encoding
        l_param.tile_size_on = 1
        l_param.cp_tdx = tile_width
        l_param.cp_tdy = tile_height

        # use irreversible encoding
        l_param.irreversible = irreversible

        l_param.numresolution = 6

        l_param.prog_order = glymur.core.LRCP

        l_params = (glymur.lib._openjp2.ImageComptParmType * num_comps)()
        for j in range(num_comps):
            l_params[j].dx = 1
            l_params[j].dy = 1
            l_params[j].h = image_height
            l_params[j].w = image_width
            l_params[j].sgnd = 0
            l_params[j].prec = comp_prec
            l_params[j].x0 = 0
            l_params[j].y0 = 0

        codec = glymur.lib._openjp2.create_compress(codec)

        glymur.lib._openjp2.set_info_handler(codec, None)
        glymur.lib._openjp2.set_warning_handler(codec, None)
        glymur.lib._openjp2.set_error_handler(codec, None)

        cspace = glymur.lib._openjp2.CLRSPC_SRGB
        l_image = glymur.lib._openjp2.image_tile_create(l_params, cspace)

        l_image.contents.x0 = 0
        l_image.contents.y0 = 0
        l_image.contents.x1 = image_width
        l_image.contents.y1 = image_height
        l_image.contents.color_space = glymur.lib._openjp2.CLRSPC_SRGB

        glymur.lib._openjp2.setup_encoder(codec, l_param, l_image)

        x = (filename, False)
        stream = glymur.lib._openjp2.stream_create_default_file_stream_v3(*x)
        glymur.lib._openjp2.start_compress(codec, l_image, stream)

        for j in np.arange(num_tiles):
            glymur.lib._openjp2.write_tile(codec, j, data, tile_size, stream)

        glymur.lib._openjp2.end_compress(codec, stream)
        glymur.lib._openjp2.stream_destroy_v3(stream)
        glymur.lib._openjp2.destroy_codec(codec)
        glymur.lib._openjp2.image_destroy(l_image)

    def tte0_setup(self, filename):
        kwargs = {'filename': filename,
                  'codec': glymur.lib._openjp2.CODEC_J2K,
                  'comp_prec': 8,
                  'irreversible': 1,
                  'num_comps': 3,
                  'image_height': 200,
                  'image_width': 200,
                  'tile_height': 100,
                  'tile_width': 100}
        self.tile_encoder(**kwargs)

    def test_tte0(self):
        # Runs test designated tte0 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            self.tte0_setup(tfile.name)

    def test_ttd0(self):
        # Runs test designated ttd0 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:

            # Produce the tte0 output file for ttd0 input.
            self.tte0_setup(tfile.name)

            kwargs = {'x0': 0,
                      'y0': 0,
                      'x1': 1000,
                      'y1': 1000,
                      'filename': tfile.name,
                      'codec_format': glymur.lib._openjp2.CODEC_J2K}
            self.tile_decoder(**kwargs)

    def tte1_setup(self, filename):
        kwargs = {'filename': filename,
                  'codec': glymur.lib._openjp2.CODEC_J2K,
                  'comp_prec': 8,
                  'irreversible': 1,
                  'num_comps': 3,
                  'image_height': 256,
                  'image_width': 256,
                  'tile_height': 128,
                  'tile_width': 128}
        self.tile_encoder(**kwargs)

    def test_tte1(self):
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            # Runs test designated tte1 in OpenJPEG test suite.
            self.tte1_setup(tfile.name)

    def test_ttd1(self):
        # Runs test designated ttd1 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:

            # Produce the tte0 output file for ttd0 input.
            self.tte1_setup(tfile.name)

            kwargs = {'x0': 0,
                      'y0': 0,
                      'x1': 128,
                      'y1': 128,
                      'filename': tfile.name,
                      'codec_format': glymur.lib._openjp2.CODEC_J2K}
            self.tile_decoder(**kwargs)

    def test_rta1(self):
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            # Runs test designated rta1 in OpenJPEG test suite.
            self.tte1_setup(tfile.name)

            kwargs = {'codec_format':  glymur.lib._openjp2.CODEC_J2K}
            self.j2k_random_tile_access(tfile.name, **kwargs)

    def tte2_setup(self, filename):
        kwargs = {'filename': filename,
                  'codec': glymur.lib._openjp2.CODEC_JP2,
                  'comp_prec': 8,
                  'irreversible': 1,
                  'num_comps': 3,
                  'image_height': 256,
                  'image_width': 256,
                  'tile_height': 128,
                  'tile_width': 128}
        self.tile_encoder(**kwargs)

    def test_tte2(self):
        # Runs test designated tte2 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            self.tte2_setup(tfile.name)

    def test_ttd2(self):
        # Runs test designated ttd2 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            # Produce the tte0 output file for ttd0 input.
            self.tte2_setup(tfile.name)

            kwargs = {'x0': 0,
                      'y0': 0,
                      'x1': 128,
                      'y1': 128,
                      'filename': tfile.name,
                      'codec_format': glymur.lib._openjp2.CODEC_JP2}
            self.tile_decoder(**kwargs)

    def test_rta2(self):
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            # Runs test designated rta2 in OpenJPEG test suite.
            self.tte2_setup(tfile.name)

            kwargs = {'codec_format':  glymur.lib._openjp2.CODEC_JP2}
            self.j2k_random_tile_access(tfile.name, **kwargs)

    def tte3_setup(self, filename):
        kwargs = {'filename': filename,
                  'codec': glymur.lib._openjp2.CODEC_J2K,
                  'comp_prec': 8,
                  'irreversible': 1,
                  'num_comps': 1,
                  'image_height': 256,
                  'image_width': 256,
                  'tile_height': 128,
                  'tile_width': 128}
        self.tile_encoder(**kwargs)

    def test_tte3(self):
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            # Runs test designated tte3 in OpenJPEG test suite.
            self.tte3_setup(tfile.name)

    def test_rta3(self):
        # Runs test designated rta3 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            self.tte3_setup(tfile.name)

            kwargs = {'codec_format':  glymur.lib._openjp2.CODEC_J2K}
            self.j2k_random_tile_access(tfile.name, **kwargs)

    def tte4_setup(self, filename):
        kwargs = {'filename': filename,
                  'codec': glymur.lib._openjp2.CODEC_J2K,
                  'comp_prec': 8,
                  'irreversible': 0,
                  'num_comps': 1,
                  'image_height': 256,
                  'image_width': 256,
                  'tile_height': 128,
                  'tile_width': 128}
        self.tile_encoder(**kwargs)

    def test_tte4(self):
        # Runs test designated tte4 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            self.tte4_setup(tfile.name)

    def test_rta4(self):
        # Runs test designated rta4 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            self.tte4_setup(tfile.name)

            kwargs = {'codec_format':  glymur.lib._openjp2.CODEC_J2K}
            self.j2k_random_tile_access(tfile.name, **kwargs)

    def tte5_setup(self, filename):
        kwargs = {'filename': filename,
                  'codec': glymur.lib._openjp2.CODEC_J2K,
                  'comp_prec': 8,
                  'irreversible': 0,
                  'num_comps': 1,
                  'image_height': 512,
                  'image_width': 512,
                  'tile_height': 256,
                  'tile_width': 256}
        self.tile_encoder(**kwargs)

    def test_tte5(self):
        # Runs test designated tte5 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            self.tte5_setup(tfile.name)

    def test_rta5(self):
        # Runs test designated rta5 in OpenJPEG test suite.
        with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
            self.tte5_setup(tfile.name)

            kwargs = {'codec_format':  glymur.lib._openjp2.CODEC_J2K}
            self.j2k_random_tile_access(tfile.name, **kwargs)

if __name__ == "__main__":
    unittest.main()



#pylint:  disable-all
import os
import pkg_resources
import re
import sys
import tempfile
import warnings

if sys.hexversion < 0x03000000:
    from StringIO import StringIO
else:
    from io import StringIO

if sys.hexversion < 0x02070000:
    import unittest2 as unittest
else:
    import unittest

import glymur


@unittest.skipIf(glymur.lib.openjp2.OPENJP2 is None,
                 "Missing openjp2 library.")
class TestCallbacks(unittest.TestCase):

    def setUp(self):
        # Save sys.stdout.
        self.stdout = sys.stdout
        sys.stdout = StringIO()
        self.jp2file = glymur.data.nemo()
        self.j2kfile = glymur.data.goodstuff()

    def tearDown(self):
        # Restore stdout.
        sys.stdout = self.stdout

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_info_callback_on_write(self):
        # Verify the messages printed when writing an image in verbose mode.
        j = glymur.Jp2k(self.jp2file)
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            tiledata = j.read(tile=0)
        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
            j = glymur.Jp2k(tfile.name, 'wb')
            j.write(tiledata, verbose=True)
        actual = sys.stdout.getvalue().strip()
        expected = '[INFO] tile number 1 / 1'
        self.assertEqual(actual, expected)

    def test_info_warning_callbacks_on_read(self):
        # Verify that we get the expected stdio output when our internal info
        # callback handler is enabled.
        j = glymur.Jp2k(self.j2kfile)
        d = j.read(rlevel=1, verbose=True, area=(0, 0, 200, 150))
        actual = sys.stdout.getvalue().strip()

        lines = ['[INFO] Start to read j2k main header (0).',
                 '[INFO] Main header has been correctly decoded.',
                 '[INFO] Setting decoding area to 0,0,150,200',
                 '[INFO] Header of tile 0 / 0 has been read.',
                 '[INFO] Tile 1/1 has been decoded.',
                 '[INFO] Image data has been updated with tile 1.']

        expected = '\n'.join(lines)
        self.assertEqual(actual, expected)


@unittest.skipIf(glymur.lib.openjp2.OPENJPEG is None,
                 "Missing openjpeg library.")
class TestCallbacks15(unittest.TestCase):
    """This test suite is for OpenJPEG 1.5.1 properties.
    """

    @classmethod
    def setUpClass(cls):
        # Monkey patch the package so as to use OPENJPEG instead of OPENJP2
        cls.openjp2 = glymur.lib.openjp2.OPENJP2
        glymur.lib.openjp2.OPENJP2 = None

    @classmethod
    def tearDownClass(cls):
        # Restore OPENJP2
        glymur.lib.openjp2.OPENJP2 = cls.openjp2

    def setUp(self):
        # Save sys.stdout.
        self.stdout = sys.stdout
        sys.stdout = StringIO()
        self.jp2file = glymur.data.nemo()
        self.j2kfile = glymur.data.goodstuff()

    def tearDown(self):
        # Restore stdout.
        sys.stdout = self.stdout

    def test_info_callbacks_on_read(self):
        # Verify that we get the expected stdio output when our internal info
        # callback handler is enabled.
        j = glymur.Jp2k(self.j2kfile)
        d = j.read(rlevel=1, verbose=True)
        actual = sys.stdout.getvalue().strip()

        regex = re.compile(r"""\[INFO\]\stile\s1\sof\s1\s+
                               \[INFO\]\s-\stiers-1\stook\s
                                       [0-9]+\.[0-9]+\ss\s+
                               \[INFO\]\s-\sdwt\stook\s
                                       (-){0,1}[0-9]+\.[0-9]+\ss\s+
                               \[INFO\]\s-\stile\sdecoded\sin\s
                                       [0-9]+\.[0-9]+\ss""",
                           re.VERBOSE)
        if sys.hexversion <= 0x03020000:
            self.assertRegexpMatches(actual, regex)
        else:
            self.assertRegex(actual, regex)


if __name__ == "__main__":
    unittest.main()



#pylint:  disable-all
import doctest
import os
import sys
import tempfile
import xml.etree.cElementTree as ET

if sys.hexversion < 0x02070000:
    import unittest2 as unittest
else:
    import unittest

import numpy as np
import pkg_resources

import glymur
from glymur import Jp2k
from glymur.jp2box import *
from glymur.core import COLOR, OPACITY
from glymur.core import RED, GREEN, BLUE, GREY, WHOLE_IMAGE


# Doc tests should be run as well.
def load_tests(loader, tests, ignore):
    if os.name == "nt":
        # Can't do it on windows, temporary file issue.
        return tests
    tests.addTests(doctest.DocTestSuite('glymur.jp2box'))
    return tests


@unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
@unittest.skipIf(glymur.lib.openjp2.OPENJP2 is None,
                 "Missing openjp2 library.")
class TestChannelDefinition(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        """Need a one_plane plane image for greyscale testing."""
        j2k = Jp2k(glymur.data.goodstuff())
        data = j2k.read()
        # Write the first component back out to file.
        with tempfile.NamedTemporaryFile(suffix=".j2k", delete=False) as tfile:
            grey_j2k = Jp2k(tfile.name, 'wb')
            grey_j2k.write(data[:, :, 0])
            cls.one_plane = tfile.name
        # Write the first two components back out to file.
        with tempfile.NamedTemporaryFile(suffix=".j2k", delete=False) as tfile:
            grey_j2k = Jp2k(tfile.name, 'wb')
            grey_j2k.write(data[:, :, 0:1])
            cls.two_planes = tfile.name
        # Write four components back out to file.
        with tempfile.NamedTemporaryFile(suffix=".j2k", delete=False) as tfile:
            rgba_jp2 = Jp2k(tfile.name, 'wb')
            shape = (data.shape[0], data.shape[1], 1)
            alpha = np.zeros((shape), dtype=data.dtype)
            data4 = np.concatenate((data, alpha), axis=2)
            rgba_jp2.write(data4)
            cls.four_planes = tfile.name

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.one_plane)
        os.unlink(cls.two_planes)
        os.unlink(cls.four_planes)

    def setUp(self):
        self.jp2file = glymur.data.nemo()
        self.j2kfile = glymur.data.goodstuff()

        j2k = Jp2k(self.j2kfile)
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)

        self.jP = JPEG2000SignatureBox()
        self.ftyp = FileTypeBox()
        self.jp2h = JP2HeaderBox()
        self.jp2c = ContiguousCodestreamBox()
        self.ihdr = ImageHeaderBox(height=height, width=width,
                                   num_components=num_components)
        self.colr_rgb = ColourSpecificationBox(colorspace=glymur.core.SRGB)
        self.colr_gr = ColourSpecificationBox(colorspace=glymur.core.GREYSCALE)

    def tearDown(self):
        pass

    def test_cdef_no_inputs(self):
        """channel_type and association are required inputs."""
        with self.assertRaises(IOError):
            glymur.jp2box.ChannelDefinitionBox()

    def test_rgb_with_index(self):
        """Just regular RGB."""
        j2k = Jp2k(self.j2kfile)
        channel_type = [COLOR, COLOR, COLOR]
        association = [RED, GREEN, BLUE]
        cdef = glymur.jp2box.ChannelDefinitionBox(index=[0, 1, 2],
                                                  channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_rgb, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)

            jp2 = Jp2k(tfile.name)
            jp2h = jp2.box[2]
            boxes = [box.box_id for box in jp2h.box]
            self.assertEqual(boxes, ['ihdr', 'colr', 'cdef'])
            self.assertEqual(jp2h.box[2].index, (0, 1, 2))
            self.assertEqual(jp2h.box[2].channel_type,
                             (COLOR, COLOR, COLOR))
            self.assertEqual(jp2h.box[2].association,
                             (RED, GREEN, BLUE))

    def test_rgb(self):
        """Just regular RGB, but don't supply the optional index."""
        j2k = Jp2k(self.j2kfile)
        channel_type = [COLOR, COLOR, COLOR]
        association = [RED, GREEN, BLUE]
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_rgb, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)

            jp2 = Jp2k(tfile.name)
            jp2h = jp2.box[2]
            boxes = [box.box_id for box in jp2h.box]
            self.assertEqual(boxes, ['ihdr', 'colr', 'cdef'])
            self.assertEqual(jp2h.box[2].index, (0, 1, 2))
            self.assertEqual(jp2h.box[2].channel_type,
                             (COLOR, COLOR, COLOR))
            self.assertEqual(jp2h.box[2].association,
                             (RED, GREEN, BLUE))

    def test_rgba(self):
        """Just regular RGBA."""
        j2k = Jp2k(self.four_planes)
        channel_type = (COLOR, COLOR, COLOR, OPACITY)
        association = (RED, GREEN, BLUE, WHOLE_IMAGE)
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_rgb, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)

            jp2 = Jp2k(tfile.name)
            jp2h = jp2.box[2]
            boxes = [box.box_id for box in jp2h.box]
            self.assertEqual(boxes, ['ihdr', 'colr', 'cdef'])
            self.assertEqual(jp2h.box[2].index, (0, 1, 2, 3))
            self.assertEqual(jp2h.box[2].channel_type, channel_type)
            self.assertEqual(jp2h.box[2].association, association)

    def test_bad_rgba(self):
        """R, G, and B must be specified."""
        j2k = Jp2k(self.four_planes)
        channel_type = (COLOR, COLOR, OPACITY, OPACITY)
        association = (RED, GREEN, BLUE, WHOLE_IMAGE)
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_rgb, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError) as ce:
                j2k.wrap(tfile.name, boxes=boxes)

    def test_grey(self):
        """Just regular greyscale."""
        j2k = Jp2k(self.one_plane)
        channel_type = (COLOR,)
        association = (GREY,)
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_gr, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)

            jp2 = Jp2k(tfile.name)
            jp2h = jp2.box[2]
            boxes = [box.box_id for box in jp2h.box]
            self.assertEqual(boxes, ['ihdr', 'colr', 'cdef'])
            self.assertEqual(jp2h.box[2].index, (0,))
            self.assertEqual(jp2h.box[2].channel_type, channel_type)
            self.assertEqual(jp2h.box[2].association, association)

    def test_grey_alpha(self):
        """Just regular greyscale plus alpha."""
        j2k = Jp2k(self.two_planes)
        channel_type = (COLOR, OPACITY)
        association = (GREY, WHOLE_IMAGE)
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_gr, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)

            jp2 = Jp2k(tfile.name)
            jp2h = jp2.box[2]
            boxes = [box.box_id for box in jp2h.box]
            self.assertEqual(boxes, ['ihdr', 'colr', 'cdef'])
            self.assertEqual(jp2h.box[2].index, (0, 1))
            self.assertEqual(jp2h.box[2].channel_type, channel_type)
            self.assertEqual(jp2h.box[2].association, association)

    def test_bad_grey_alpha(self):
        """A greyscale image with alpha layer must specify a color channel"""
        j2k = Jp2k(self.two_planes)

        channel_type = (OPACITY, OPACITY)
        association = (GREY, WHOLE_IMAGE)

        # This cdef box
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)
        boxes = [self.ihdr, self.colr_gr, cdef]
        self.jp2h.box = boxes
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises((OSError, IOError)) as ce:
                j2k.wrap(tfile.name, boxes=boxes)

    def test_only_one_cdef_in_jp2_header(self):
        """There can only be one channel definition box in the jp2 header."""
        j2k = Jp2k(self.j2kfile)

        channel_type = (COLOR, COLOR, COLOR)
        association = (RED, GREEN, BLUE)
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)

        boxes = [self.ihdr, cdef, self.colr_rgb, cdef]
        self.jp2h.box = boxes

        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]

        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

    def test_not_in_jp2_header(self):
        j2k = Jp2k(self.j2kfile)
        boxes = [self.ihdr, self.colr_rgb]
        self.jp2h.box = boxes

        channel_type = (COLOR, COLOR, COLOR)
        association = (RED, GREEN, BLUE)
        cdef = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                  association=association)

        boxes = [self.jP, self.ftyp, self.jp2h, cdef, self.jp2c]

        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

    def test_bad_type(self):
        # Channel types are limited to 0, 1, 2, 65535
        # Should reject if not all of index, channel_type, association the
        # same length.
        channel_type = (COLOR, COLOR, 3)
        association = (RED, GREEN, BLUE)
        with self.assertRaises(IOError):
            box = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                     association=association)

    def test_wrong_lengths(self):
        # Should reject if not all of index, channel_type, association the
        # same length.
        channel_type = (COLOR, COLOR)
        association = (RED, GREEN, BLUE)
        with self.assertRaises(IOError):
            box = glymur.jp2box.ChannelDefinitionBox(channel_type=channel_type,
                                                     association=association)


@unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
class TestXML(unittest.TestCase):

    def setUp(self):
        self.jp2file = glymur.data.nemo()
        self.j2kfile = glymur.data.goodstuff()

        raw_xml = b"""<?xml version="1.0"?>
        <data>
            <country name="Liechtenstein">
                <rank>1</rank>
                <year>2008</year>
                <gdppc>141100</gdppc>
                <neighbor name="Austria" direction="E"/>
                <neighbor name="Switzerland" direction="W"/>
            </country>
            <country name="Singapore">
                <rank>4</rank>
                <year>2011</year>
                <gdppc>59900</gdppc>
                <neighbor name="Malaysia" direction="N"/>
            </country>
            <country name="Panama">
                <rank>68</rank>
                <year>2011</year>
                <gdppc>13600</gdppc>
                <neighbor name="Costa Rica" direction="W"/>
                <neighbor name="Colombia" direction="E"/>
            </country>
        </data>"""
        with tempfile.NamedTemporaryFile(suffix=".xml", delete=False) as tfile:
            tfile.write(raw_xml)
            tfile.flush()
        self.xmlfile = tfile.name

        j2k = Jp2k(self.j2kfile)
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)

        self.jP = JPEG2000SignatureBox()
        self.ftyp = FileTypeBox()
        self.jp2h = JP2HeaderBox()
        self.jp2c = ContiguousCodestreamBox()
        self.ihdr = ImageHeaderBox(height=height, width=width,
                                   num_components=num_components)
        self.colr = ColourSpecificationBox(colorspace=glymur.core.SRGB)

    def tearDown(self):
        os.unlink(self.xmlfile)
        pass

    def test_negative_both_file_and_xml_provided(self):
        """The XML should come from only one source."""
        j2k = Jp2k(self.j2kfile)
        xml_object = ET.parse(self.xmlfile)
        with self.assertRaises((IOError, OSError)) as ce:
            xmlb = glymur.jp2box.XMLBox(filename=self.xmlfile, xml=xml_object)

    @unittest.skipIf(os.name == "nt",
                     "Problems using NamedTemporaryFile on windows.")
    def test_basic_xml(self):
        # Should be able to write an XMLBox.
        j2k = Jp2k(self.j2kfile)

        self.jp2h.box = [self.ihdr, self.colr]

        the_xml = ET.fromstring('<?xml version="1.0"?><data>0</data>')
        xmlb = glymur.jp2box.XMLBox(xml=the_xml)
        self.assertEqual(ET.tostring(xmlb.xml),
                         b'<data>0</data>')

        boxes = [self.jP, self.ftyp, self.jp2h, xmlb, self.jp2c]

        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)
            jp2 = Jp2k(tfile.name)
            self.assertEqual(jp2.box[3].box_id, 'xml ')
            self.assertEqual(ET.tostring(jp2.box[3].xml.getroot()),
                             b'<data>0</data>')

    @unittest.skipIf(os.name == "nt",
                     "Problems using NamedTemporaryFile on windows.")
    def test_xml_from_file(self):
        j2k = Jp2k(self.j2kfile)

        self.jp2h.box = [self.ihdr, self.colr]

        xmlb = glymur.jp2box.XMLBox(filename=self.xmlfile)
        boxes = [self.jP, self.ftyp, self.jp2h, xmlb, self.jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)
            jp2 = Jp2k(tfile.name)

            output_boxes = [box.box_id for box in jp2.box]
            self.assertEqual(output_boxes, ['jP  ', 'ftyp', 'jp2h', 'xml ',
                                            'jp2c'])

            elts = jp2.box[3].xml.findall('country')
            self.assertEqual(len(elts), 3)

            neighbor = elts[1].find('neighbor')
            self.assertEqual(neighbor.attrib['name'], 'Malaysia')
            self.assertEqual(neighbor.attrib['direction'], 'N')


class TestColourSpecificationBox(unittest.TestCase):

    def setUp(self):
        self.j2kfile = glymur.data.goodstuff()

        j2k = Jp2k(self.j2kfile)
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)

        self.jP = JPEG2000SignatureBox()
        self.ftyp = FileTypeBox()
        self.jp2h = JP2HeaderBox()
        self.jp2c = ContiguousCodestreamBox()
        self.ihdr = ImageHeaderBox(height=height, width=width,
                                   num_components=num_components)

    def tearDown(self):
        pass

    @unittest.skipIf(os.name == "nt",
                     "Problems using NamedTemporaryFile on windows.")
    def test_color_specification_box_with_out_enumerated_colorspace(self):
        j2k = Jp2k(self.j2kfile)

        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        boxes[2].box = [self.ihdr, ColourSpecificationBox(colorspace=None)]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(NotImplementedError):
                j2k.wrap(tfile.name, boxes=boxes)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_missing_colr_box(self):
        j2k = Jp2k(self.j2kfile)
        boxes = [self.jP, self.ftyp, self.jp2h, self.jp2c]
        boxes[2].box = [self.ihdr]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

    def test_default_ColourSpecificationBox(self):
        b = glymur.jp2box.ColourSpecificationBox(colorspace=glymur.core.SRGB)
        self.assertEqual(b.method,  glymur.core.ENUMERATED_COLORSPACE)
        self.assertEqual(b.precedence, 0)
        self.assertEqual(b.approximation, 0)
        self.assertEqual(b.colorspace, glymur.core.SRGB)
        self.assertIsNone(b.icc_profile)

    def test_ColourSpecificationBox_with_colorspace_and_icc(self):
        # Colour specification boxes can't have both.
        with self.assertRaises((OSError, IOError)):
            colorspace = glymur.core.SRGB
            icc_profile = b'\x01\x02\x03\x04'
            b = glymur.jp2box.ColourSpecificationBox(colorspace=colorspace,
                                                     icc_profile=icc_profile)

    def test_ColourSpecificationBox_with_bad_method(self):
        colorspace = glymur.core.SRGB
        method = -1
        with self.assertRaises(IOError):
            b = glymur.jp2box.ColourSpecificationBox(colorspace=colorspace,
                                                     method=method)

    def test_ColourSpecificationBox_with_bad_approximation(self):
        colorspace = glymur.core.SRGB
        approx = -1
        with self.assertRaises(IOError):
            b = glymur.jp2box.ColourSpecificationBox(colorspace=colorspace,
                                                     approximation=approx)


@unittest.skipIf(glymur.lib.openjp2.OPENJP2 is None,
                 "Missing openjp2 library.")
class TestJp2Boxes(unittest.TestCase):

    def setUp(self):
        self.j2kfile = glymur.data.goodstuff()
        self.jp2file = glymur.data.nemo()

    def tearDown(self):
        pass

    def test_default_JPEG2000SignatureBox(self):
        # Should be able to instantiate a JPEG2000SignatureBox
        b = glymur.jp2box.JPEG2000SignatureBox()
        self.assertEqual(b.signature, (13, 10, 135, 10))

    def test_default_FileTypeBox(self):
        # Should be able to instantiate a FileTypeBox
        b = glymur.jp2box.FileTypeBox()
        self.assertEqual(b.brand, 'jp2 ')
        self.assertEqual(b.minor_version, 0)
        self.assertEqual(b.compatibility_list, ['jp2 '])

    def test_default_ImageHeaderBox(self):
        # Should be able to instantiate an image header box.
        b = glymur.jp2box.ImageHeaderBox(height=512, width=256,
                                         num_components=3)
        self.assertEqual(b.height,  512)
        self.assertEqual(b.width,  256)
        self.assertEqual(b.num_components,  3)
        self.assertEqual(b.bits_per_component, 8)
        self.assertFalse(b.signed)
        self.assertFalse(b.colorspace_unknown)

    def test_default_JP2HeaderBox(self):
        b1 = JP2HeaderBox()
        b1.box = [ImageHeaderBox(height=512, width=256),
                  ColourSpecificationBox(colorspace=glymur.core.GREYSCALE)]

    def test_default_ContiguousCodestreamBox(self):
        b = ContiguousCodestreamBox()
        self.assertEqual(b.box_id, 'jp2c')
        self.assertIsNone(b.main_header)

    def verify_wrapped_raw(self, jp2file):
        # Shared method by at least two tests.
        jp2 = Jp2k(jp2file)
        self.assertEqual(len(jp2.box), 4)

        self.assertEqual(jp2.box[0].box_id, 'jP  ')
        self.assertEqual(jp2.box[0].offset, 0)
        self.assertEqual(jp2.box[0].length, 12)
        self.assertEqual(jp2.box[0].longname, 'JPEG 2000 Signature')

        self.assertEqual(jp2.box[1].box_id, 'ftyp')
        self.assertEqual(jp2.box[1].offset, 12)
        self.assertEqual(jp2.box[1].length, 20)
        self.assertEqual(jp2.box[1].longname, 'File Type')

        self.assertEqual(jp2.box[2].box_id, 'jp2h')
        self.assertEqual(jp2.box[2].offset, 32)
        self.assertEqual(jp2.box[2].length, 45)
        self.assertEqual(jp2.box[2].longname, 'JP2 Header')

        self.assertEqual(jp2.box[3].box_id, 'jp2c')
        self.assertEqual(jp2.box[3].offset, 77)
        self.assertEqual(jp2.box[3].length, 115228)

        # jp2h super box
        self.assertEqual(len(jp2.box[2].box), 2)

        self.assertEqual(jp2.box[2].box[0].box_id, 'ihdr')
        self.assertEqual(jp2.box[2].box[0].offset, 40)
        self.assertEqual(jp2.box[2].box[0].length, 22)
        self.assertEqual(jp2.box[2].box[0].longname, 'Image Header')
        self.assertEqual(jp2.box[2].box[0].height, 800)
        self.assertEqual(jp2.box[2].box[0].width, 480)
        self.assertEqual(jp2.box[2].box[0].num_components, 3)
        self.assertEqual(jp2.box[2].box[0].bits_per_component, 8)
        self.assertEqual(jp2.box[2].box[0].signed, False)
        self.assertEqual(jp2.box[2].box[0].compression, 7)
        self.assertEqual(jp2.box[2].box[0].colorspace_unknown, False)
        self.assertEqual(jp2.box[2].box[0].ip_provided, False)

        self.assertEqual(jp2.box[2].box[1].box_id, 'colr')
        self.assertEqual(jp2.box[2].box[1].offset, 62)
        self.assertEqual(jp2.box[2].box[1].length, 15)
        self.assertEqual(jp2.box[2].box[1].longname, 'Colour Specification')
        self.assertEqual(jp2.box[2].box[1].precedence, 0)
        self.assertEqual(jp2.box[2].box[1].approximation, 0)
        self.assertEqual(jp2.box[2].box[1].colorspace, glymur.core.SRGB)
        self.assertIsNone(jp2.box[2].box[1].icc_profile)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_wrap(self):
        j2k = Jp2k(self.j2kfile)
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name)
            self.verify_wrapped_raw(tfile.name)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_wrap_jp2(self):
        j2k = Jp2k(self.jp2file)
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            jp2 = j2k.wrap(tfile.name)
        boxes = [box.box_id for box in jp2.box]
        self.assertEqual(boxes, ['jP  ', 'ftyp', 'jp2h', 'jp2c'])

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_default_layout_but_with_specified_boxes(self):
        j2k = Jp2k(self.j2kfile)
        boxes = [JPEG2000SignatureBox(),
                 FileTypeBox(),
                 JP2HeaderBox(),
                 ContiguousCodestreamBox()]
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)
        boxes[2].box = [ImageHeaderBox(height=height,
                                       width=width,
                                       num_components=num_components),
                        ColourSpecificationBox(colorspace=glymur.core.SRGB)]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            j2k.wrap(tfile.name, boxes=boxes)
            self.verify_wrapped_raw(tfile.name)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_image_header_box_not_first_in_jp2_header(self):
        # The specification says that ihdr must be the first box in jp2h.
        j2k = Jp2k(self.j2kfile)
        boxes = [JPEG2000SignatureBox(),
                 FileTypeBox(),
                 JP2HeaderBox(),
                 ContiguousCodestreamBox()]
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)
        boxes[2].box = [ColourSpecificationBox(colorspace=glymur.core.SRGB),
                        ImageHeaderBox(height=height,
                                       width=width,
                                       num_components=num_components)]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_first_2_boxes_not_jP_and_ftyp(self):
        j2k = Jp2k(self.j2kfile)
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)

        jP = JPEG2000SignatureBox()
        ftyp = FileTypeBox()
        jp2h = JP2HeaderBox()
        jp2c = ContiguousCodestreamBox()
        colr = ColourSpecificationBox(colorspace=glymur.core.SRGB)
        ihdr = ImageHeaderBox(height=height, width=width,
                              num_components=num_components)
        jp2h.box = [ihdr, colr]
        boxes = [ftyp, jP, jp2h, jp2c]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_jp2h_not_preceeding_jp2c(self):
        j2k = Jp2k(self.j2kfile)
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)

        jP = JPEG2000SignatureBox()
        ftyp = FileTypeBox()
        jp2h = JP2HeaderBox()
        jp2c = ContiguousCodestreamBox()
        colr = ColourSpecificationBox(colorspace=glymur.core.SRGB)
        ihdr = ImageHeaderBox(height=height, width=width,
                              num_components=num_components)
        jp2h.box = [ihdr, colr]
        boxes = [jP, ftyp, jp2c, jp2h]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
    def test_missing_codestream(self):
        j2k = Jp2k(self.j2kfile)
        c = j2k.get_codestream()
        height = c.segment[1].ysiz
        width = c.segment[1].xsiz
        num_components = len(c.segment[1].xrsiz)

        jP = JPEG2000SignatureBox()
        ftyp = FileTypeBox()
        jp2h = JP2HeaderBox()
        ihdr = ImageHeaderBox(height=height, width=width,
                              num_components=num_components)
        jp2h.box = [ihdr]
        boxes = [jP, ftyp, jp2h]
        with tempfile.NamedTemporaryFile(suffix=".jp2") as tfile:
            with self.assertRaises(IOError):
                j2k.wrap(tfile.name, boxes=boxes)

if __name__ == "__main__":
    unittest.main()



#pylint:  disable-all
import ctypes
import re
import sys

if sys.hexversion < 0x02070000:
    import unittest2 as unittest
else:
    import unittest

import glymur


@unittest.skipIf(glymur.lib._openjpeg.OPENJPEG is None,
                 "Missing openjpeg library.")
class TestOpenJPEG(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_version(self):
        version = glymur.lib._openjpeg.version()
        regex = re.compile('1.[345].[0-9]')
        if sys.hexversion <= 0x03020000:
            self.assertRegexpMatches(version, regex)
        else:
            self.assertRegex(version, regex)

    def test_set_default_decoder_parameters(self):
        # Verify that we properly set the default decode parameters.
        version = glymur.lib._openjpeg.version()
        minor = int(version.split('.')[1])

        dp = glymur.lib._openjpeg.DecompressionParametersType()
        glymur.lib._openjpeg.set_default_decoder_parameters(ctypes.byref(dp))

        self.assertEqual(dp.cp_reduce, 0)
        self.assertEqual(dp.cp_layer, 0)
        self.assertEqual(dp.infile, b'')
        self.assertEqual(dp.outfile, b'')
        self.assertEqual(dp.decod_format, -1)
        self.assertEqual(dp.cod_format, -1)
        self.assertEqual(dp.jpwl_correct, 0)
        self.assertEqual(dp.jpwl_exp_comps, 0)
        self.assertEqual(dp.jpwl_max_tiles, 0)
        self.assertEqual(dp.cp_limit_decoding, 0)
        if minor > 4:
            # Introduced in 1.5.x
            self.assertEqual(dp.flags, 0)
